using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using Props.Shop.Adafruit.Api;
using Props.Shop.Framework;

namespace Props.Shop.Adafruit.Tests.Api
{
    public class FakeProductListingManager : IProductListingManager
    {
        private bool disposedValue;
        public DateTime? LastDownload { get; private set; }
        private ProductListingsParser parser = new ProductListingsParser();
        private readonly ConcurrentDictionary<string, ProductListing> activeProductListingUrls = new ConcurrentDictionary<string, ProductListing>();


        public Task<IReadOnlyDictionary<string, IList<ProductListing>>> ProductListings { get; private set; }

        public async Task<ProductListing> GetProductListingFromIdentifier(string url)
        {
            if (disposedValue) throw new ObjectDisposedException("ProductListingManager");
            await ProductListings;
            return activeProductListingUrls[url];
        }

        public void RefreshProductListings()
        {
            if (disposedValue) throw new ObjectDisposedException("ProductListingManager");
            if ((LastDownload != null && DateTime.UtcNow - LastDownload <= TimeSpan.FromMilliseconds(5 * 60 * 1000)) || (ProductListings != null && !ProductListings.IsCompleted)) return;
            ProductListings = DownloadListings();
        }

        private Task<IReadOnlyDictionary<string, IList<ProductListing>>> DownloadListings()
        {
            if (disposedValue) throw new ObjectDisposedException("ProductListingManager");
            LastDownload = DateTime.UtcNow;
            using (Stream stream = File.OpenRead("./Assets/products.json"))
            {
                parser.BuildProductListings(stream);

            }
            Dictionary<string, IList<ProductListing>> listingNames = new Dictionary<string, IList<ProductListing>>();
            activeProductListingUrls.Clear();
            foreach (ProductListing product in parser.ProductListings)
            {
                activeProductListingUrls.TryAdd(product.URL, product);
                IList<ProductListing> sameProducts = listingNames.GetValueOrDefault(product.Name);
                if (sameProducts == null)
                {
                    sameProducts = new List<ProductListing>();
                    listingNames.Add(product.Name, sameProducts);
                }

                sameProducts.Add(product);
            }
            return Task.FromResult<IReadOnlyDictionary<string, IList<ProductListing>>>(listingNames);
        }

        public void StartUpdateTimer(int delay = 300000, int period = 300000)
        {
            RefreshProductListings();
        }

        public void StopUpdateTimer()
        {
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!disposedValue)
            {
                if (disposing)
                {
                }

                disposedValue = true;
            }
        }

        public void Dispose()
        {
            Dispose(disposing: true);
            GC.SuppressFinalize(this);
        }
    }
}